/******************************************************************************
  Copyright (c)  2001-2007 by Turku PET Centre

  mathfunc.h

  Version:
  2001-10-04 Vesa Oikonen
  2003-02-19 VO
  2003-08-14 VO
  2004-09-20 VO
    Changed the comments to doxygen style.
  2005-10-15 VO
    Changed function names.
    Field "name" added to Voi struct.
  2007-08-12 VO
    Added define for BACKUP_EXTENSION.  
  2007-08-13 VO
    Units length in DFT struct is defined as MAX_UNITS_LEN+1 (previously 13).
  2007-09-13 VO
    Added field fitmethod in FIT struct.

******************************************************************************/
#ifndef _MATHFUNC_H
#define _MATHFUNC_H
/*****************************************************************************/
#include <time.h>
/*****************************************************************************/
#include "studynr.h"
#include "petunits.h"
/*****************************************************************************/
#define FIT_VER "FIT1"
/*****************************************************************************/
/* Missing data */
#ifndef NA
#define NA -99000.
#endif
/*****************************************************************************/
/* Max nr of parameters */
#ifndef MAX_FITPARAMS
#define MAX_FITPARAMS 20
#endif
/*****************************************************************************/
/* Backup file extension */
#ifndef BACKUP_EXTENSION
#define BACKUP_EXTENSION ".bak"
#endif 
/*****************************************************************************/
int MATHFUNC_TEST;
/*****************************************************************************/
char fiterrmsg[64];
/*****************************************************************************/
/*
 *  Data structure definitions
 */

/** Definitions for one curve */
typedef struct {
  /** Name of the curve */
  /** Name of region, including hemispehere, plane etc;
      this will replace the voiname, hemisphere and place. */
  char      name[MAX_REGIONNAME_LEN+1];
  char      voiname[7], hemisphere[7], place[7];
  /** Number (type) of function */
  int       type;
  /** The number of parameters */
  int       parNr;
  /** Fit start time */
  double    start;
  /** Fit end time */
  double    end;
  /** Number of data points in the fit */
  int       dataNr;
  /** Fitted parameters */
  double    p[MAX_FITPARAMS];
  /** (weighted) sum-of-squares */
  double    wss;
  /** Temporary switches for outside procedures */
  char      sw, sw2, sw3;
} FitVOI;

/** Definitions for a set of curves */
typedef struct {
  /** Number of regions */
  int       voiNr;
  /** Name of original datafile */
  char      datafile[1024];
  /** Unit of concentration */
  char      unit[MAX_UNITS_LEN+1];
  /** Time unit: TUNIT_UNKNOWN, TUNIT_SEC, TUNIT_MIN, ... */
  int       timeunit;
  /** Pointers to regional curves */
  FitVOI   *voi;
  /** Fit date and time */
  time_t    time;
  /** Program name */
  char      program[1024];
  /** Internal variables: Number of allocated curves (VOIs) */
  int      _voidataNr; /*  */
} FIT;
/*****************************************************************************/
extern void fitEmpty(FIT *fit);
extern void fitInit(FIT *fit);
extern int fitSetmem(FIT *fit, int voiNr);
extern void fitPrint(FIT *fit);
extern int fitWrite(FIT *fit, char *filename);
extern int fitRead(char *filename, FIT *fit);
extern int fitEval(FitVOI *r, double x, double *y);
extern int fitEvaltac(FitVOI *r, double *x, double *y, int dataNr);
extern int fitFunctionname(int type, char *str);
extern int fitFunctionformat(int type, char *str);
extern int fitIntegralEval(FitVOI *r, double x, double *yi);
extern int fitIntegralEvaltac(FitVOI *r, double *x, double *yi, int dataNr);
extern int fitDerivEval(FitVOI *r, double x, double *yd);
extern int fitDerivEvaltac(FitVOI *r, double *x, double *yd, int dataNr);
/*****************************************************************************/
/* Deprecated functions. Please don't use these anymore */
#define emptyFIT fitEmpty
#define initFIT fitInit
#define setmemFIT fitSetmem
#define printFIT fitPrint
#define writeFIT fitWrite
#define readFIT fitRead
#define evalFIT fitEval
#define evaltacFIT fitEvaltac
#define functionnameFIT fitFunctionname
#define functionformatFIT fitFunctionformat
#define ievalFIT fitIntegralEval
#define ievaltacFIT fitIntegralEvaltac
#define devalFIT fitDerivEval
#define devaltacFIT fitDerivEvaltac
/*****************************************************************************/
#endif

